"""
Classifies: CHEBI:133241 omega-carboxyacyl-CoA(5-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_omega_carboxyacyl_CoA_5__(smiles: str):
    """
    Determines if a molecule is an omega-carboxyacyl-CoA(5-) based on structure.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an omega-carboxyacyl-CoA(5-), False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for CoA core structure components
    # - Adenine 
    adenine_pattern = Chem.MolFromSmarts('n1cnc2c(N)ncnc12')
    if not mol.HasSubstructMatch(adenine_pattern):
        return False, "Missing adenine moiety"

    # - Ribose with 3' phosphate and diphosphate
    ribose_diphosphate = Chem.MolFromSmarts('COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12')
    if not mol.HasSubstructMatch(ribose_diphosphate):
        return False, "Missing ribose-phosphate-diphosphate moiety"
        
    # - Pantetheine
    pantetheine = Chem.MolFromSmarts('CC(C)(COP)[C@@H](O)C(=O)NCCC(=O)NCCSC=O')
    if not mol.HasSubstructMatch(pantetheine):
        return False, "Missing pantetheine moiety"

    # Check for terminal carboxylate and thioester in a specific arrangement
    terminal_pattern = Chem.MolFromSmarts('SC(=O)CCCC([O-])=O')
    if not mol.HasSubstructMatch(terminal_pattern):
        return False, "Missing correct terminal carboxylate and thioester arrangement"

    # Count total negative charges - should be 5
    negative_charges = 0
    for atom in mol.GetAtoms():
        negative_charges += atom.GetFormalCharge()
    if negative_charges != -5:
        return False, f"Total negative charge is {negative_charges}, should be -5"

    # Check for absence of certain problematic features
    # No double bonds in the chain between thioester and terminal carboxylate
    problematic_pattern = Chem.MolFromSmarts('SC(=O)C=CC([O-])=O')
    if mol.HasSubstructMatch(problematic_pattern):
        return False, "Contains unsaturated bonds in the acyl chain"

    return True, "Contains all required structural components of omega-carboxyacyl-CoA(5-)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:133241',
                          'name': 'omega-carboxyacyl-CoA(5-)',
                          'definition': 'An acyl-CoA oxoanion obtained by '
                                        'deprotonation of the phosphate, '
                                        'diphosphate and carboxy groups of any '
                                        'omega-carboxyacyl-CoA; major species '
                                        'at pH 7.3.',
                          'parents': ['CHEBI:58946']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.03448275862068965 is too low.\n'
               'True positives: '
               "[('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)')]\n"
               'False positives: '
               "[('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@H](O)CCCCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)C(O)C(=O)NCCC(=O)NCCSC(=O)C(\\\\S)=C\\\\CC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)=CCC\\\\C(CC([O-])=O)=C/C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)C(O)C(=O)NCCC(=O)NCCSC(=O)C(=O)CCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N1C=NC2=C1N=CN=C2N)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)[C@@H](CC([O-])=O)[C@@H](O)C1=CC=CC=C1', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C(O)CCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C(\\\\CC([O-])=O)=C\\\\c1ccccc1', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(CCC(C(CCC([O-])=O)=O)C)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CCCCCCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@H](O)CCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[H]C(CCC([O-])=O)=C([H])C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)CC(C(=O)[O-])C=4C=CC=CC4)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(C4=C([C@@H](CCC4=O)C)CCC([O-])=O)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=C)C([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@](C)(O)C([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(C)(O)C([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C=3C(=C(N=CN3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(/C=C/CCC([O-])=O)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)CC(C(=O)[O-])C=4C=CC=CC4N)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@](C)(O)C([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)CCC(=O)C=4C=CC=CC4C(=O)[O-])=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C/CCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('C[C@@H](CC([O-])=O)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@H](O)CCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)[C@@H](CC([O-])=O)Cc1ccccc1', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(O)CCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C=3C(=C(N=CN3)N)N=C2)O[C@H](COP(OP(OCC([C@H](C(NCCC(NCCSC(=O)[C@@H](CCC(=O)[O-])O)=O)=O)O)(C)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@H](O)CCCCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N1C=NC2=C1N=CN=C2N)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)[C@@H](CC([O-])=O)C(=O)C1=CC=CC=C1', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)C(O)C(=O)NCCC(=O)NCCSC(=O)C(=S)CCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)C(CC(=O)[O-])C=4C=CC=CC4N)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C\\\\C=C/CC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)[C@H](CC(=O)[O-])CC4=CC=CC=C4)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C=CCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[H]C(CC([O-])=O)=C([H])CC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CCCCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('C[C@@H]([C@@H](O)C([O-])=O)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(CC(CCC(C(CCC([O-])=O)=O)C)=O)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CCCCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)C[C@@H](O)CCC([O-])=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('C[C@](O)(CC([O-])=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)C(CC(=O)[O-])C=4C=CC=CC4)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@H](O)CCCCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C[C@@H](O)CCCCCCCCC([O-])=O', "
               "'Contains all required structural components of "
               "omega-carboxyacyl-CoA(5-)')]\n"
               'False negatives: '
               "[('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C([O-])=O', "
               "'Missing terminal carboxylate and/or thioester linkage')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 5,
    'num_true_negatives': 183910,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.16666666666666666,
    'recall': 0.5,
    'f1': 0.25,
    'accuracy': 0.9999673765883523}


"""
Classifies: CHEBI:22531 aminotoluene
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_aminotoluene(smiles: str):
    """
    Determines if a molecule is an aminotoluene (toluene with one or more amino groups).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an aminotoluene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a toluene substructure
    toluene_pattern = Chem.MolFromSmarts('c1ccc(C)cc1')
    if not mol.HasSubstructMatch(toluene_pattern):
        return False, "Toluene substructure not found"

    # Check for the presence of at least one amino group
    amino_pattern = Chem.MolFromSmarts('N')
    if not mol.HasSubstructMatch(amino_pattern):
        return False, "No amino groups found"

    # Count the number of amino groups
    amino_groups = sum(atom.GetSymbol() == 'N' for atom in mol.GetAtoms())

    if amino_groups > 0:
        return True, f"Aminotoluene with {amino_groups} amino group(s)"
    else:
        return False, "No amino groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22531',
                          'name': 'aminotoluene',
                          'definition': 'Any member of the class of  toluenes '
                                        'carrying one or more amino groups.',
                          'parents': ['CHEBI:27024', 'CHEBI:48975']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 263,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.7252747252747253}


"""
Classifies: CHEBI:145408 ketene acetal
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_ketene_acetal(smiles: str):
    """
    Determines if a molecule is a ketene acetal (RR'C=C(OR'')(OR''') where R'', R''' != H).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a ketene acetal, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all double bonds
    double_bonds = [bond for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.DOUBLE]

    # Check for at least one double bond
    if not double_bonds:
        return False, "No double bonds found"

    for bond in double_bonds:
        atom1 = bond.GetBeginAtom()
        atom2 = bond.GetEndAtom()

        # Check for carbon atoms on both ends of the double bond
        if atom1.GetSymbol() != 'C' or atom2.GetSymbol() != 'C':
            continue

        # Check for at least one alkoxy group (RO-) on each carbon
        alkoxy_groups_atom1 = 0
        alkoxy_groups_atom2 = 0
        for neighbor in atom1.GetNeighbors():
            if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 0:
                alkoxy_groups_atom1 += 1
        for neighbor in atom2.GetNeighbors():
            if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 0:
                alkoxy_groups_atom2 += 1

        if alkoxy_groups_atom1 >= 1 and alkoxy_groups_atom2 >= 1:
            return True, "Molecule is a ketene acetal"

    return False, "Molecule is not a ketene acetal"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:145408',
                          'name': 'ketene acetal',
                          'definition': 'An organooxygen compound having the '
                                        "structure RR'C=C(OR'')(OR''') where "
                                        "R'', R''' =/= H. Formally, they are "
                                        'ethers of the enolic form of esters. '
                                        'They bear the same structural '
                                        'relationship to ketenes that acetals '
                                        'bear to aldehydes and ketones.',
                          'parents': ['CHEBI:36963']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 157964,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9993610223642172}


"""
Classifies: CHEBI:28874 phosphatidylinositol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_phosphatidylinositol(smiles: str):
    """
    Determines if a molecule is a phosphatidylinositol, defined as 'Any glycerophosphoinositol having one phosphatidyl group esterified to one of the hydroxy groups of inositol.'

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a phosphatidylinositol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains an inositol ring
    inositol_ring_atoms = []
    for ring in mol.GetRingInfo().AtomRings():
        if len(ring) == 6:
            ring_atoms = [mol.GetAtomWithIdx(idx) for idx in ring]
            if all(atom.GetSymbol() == "O" for atom in ring_atoms):
                inositol_ring_atoms = ring_atoms
                break

    if not inositol_ring_atoms:
        return False, "Molecule does not contain an inositol ring"

    # Check if the inositol ring has a phosphate group attached
    phosphate_attached = False
    for atom in inositol_ring_atoms:
        for neighbor in atom.GetNeighbors():
            if neighbor.GetSymbol() == "P":
                phosphate_attached = True
                break

    if not phosphate_attached:
        return False, "Inositol ring does not have a phosphate group attached"

    # Check if the molecule contains glycerol
    glycerol_atoms = []
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == "O" and len(atom.GetNeighbors()) == 3:
            neighbors = [neighbor.GetSymbol() for neighbor in atom.GetNeighbors()]
            if "C" in neighbors and "C" in neighbors and "C" in neighbors:
                glycerol_atoms.append(atom)

    if not glycerol_atoms:
        return False, "Molecule does not contain glycerol"

    # Check if the glycerol is attached to the phosphate
    glycerol_attached = False
    for atom in glycerol_atoms:
        for neighbor in atom.GetNeighbors():
            if neighbor.GetSymbol() == "P":
                glycerol_attached = True
                break

    if not glycerol_attached:
        return False, "Glycerol is not attached to the phosphate group"

    # Check if the molecule has acyl chains
    acyl_chains = []
    for atom in glycerol_atoms:
        for neighbor in atom.GetNeighbors():
            if neighbor.GetSymbol() == "C" and len(neighbor.GetNeighbors()) > 1:
                acyl_chain = [neighbor]
                current_atom = neighbor
                while True:
                    neighbors = [n for n in current_atom.GetNeighbors() if n.GetSymbol() == "C"]
                    if len(neighbors) == 1:
                        current_atom = neighbors[0]
                        acyl_chain.append(current_atom)
                    else:
                        break
                acyl_chains.append(acyl_chain)

    if not acyl_chains or len(acyl_chains) < 2:
        return False, "Molecule does not have two acyl chains"

    # If all conditions are met, it's a phosphatidylinositol
    acyl_chain_lengths = [len(chain) for chain in acyl_chains]
    return True, f"Phosphatidylinositol with acyl chain lengths {acyl_chain_lengths}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28874',
                          'name': 'phosphatidylinositol',
                          'definition': 'Any glycerophosphoinositol having one '
                                        'phosphatidyl group esterified to one '
                                        'of the hydroxy groups of inositol.',
                          'parents': ['CHEBI:36315']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183499,
    'num_false_negatives': 48,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9997384866001624}


"""
Classifies: CHEBI:25903 peptide antibiotic
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect

def is_peptide_antibiotic(smiles: str):
    """
    Determines if a molecule is a peptide antibiotic.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a peptide antibiotic, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check molecular weight - peptide antibiotics are typically large molecules
    mw = Descriptors.ExactMolWt(mol)
    if mw < 400:
        return False, "Molecular weight too low for peptide antibiotic"

    # Count peptide bonds (amide bonds)
    amide_pattern = Chem.MolFromSmarts('[NX3][CX3](=[OX1])')
    num_amide_bonds = len(mol.GetSubstructMatches(amide_pattern))
    if num_amide_bonds < 3:
        return False, "Too few peptide bonds"

    # Count amino groups
    amino_pattern = Chem.MolFromSmarts('[NX3;H2,H1;!$(NC=O)]')
    num_amino_groups = len(mol.GetSubstructMatches(amino_pattern))
    
    # Look for basic amino acid residues (Lys, Arg, His patterns)
    basic_aa_pattern = Chem.MolFromSmarts('[NX3;H2,H1]CC[CH2]')  # Simplified pattern
    num_basic_aa = len(mol.GetSubstructMatches(basic_aa_pattern))

    # Calculate ratio of heteroatoms
    num_n = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 7)
    num_o = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 8)
    total_atoms = mol.GetNumAtoms()
    heteroatom_ratio = (num_n + num_o) / total_atoms

    # Combine criteria for classification
    if (num_amide_bonds >= 3 and  # Multiple peptide bonds
        heteroatom_ratio > 0.2 and  # High heteroatom content
        (num_amino_groups >= 2 or num_basic_aa >= 1) and  # Contains amino groups
        mw >= 400):  # Reasonable molecular weight
        
        reasons = []
        reasons.append(f"Contains {num_amide_bonds} peptide bonds")
        reasons.append(f"Molecular weight: {mw:.1f}")
        if num_amino_groups > 0:
            reasons.append(f"Contains {num_amino_groups} amino groups")
        if num_basic_aa > 0:
            reasons.append(f"Contains basic amino acid residues")
        reasons.append(f"Heteroatom ratio: {heteroatom_ratio:.2f}")
        
        return True, "; ".join(reasons)

    return False, "Does not meet structural criteria for peptide antibiotic"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25903',
                          'name': 'peptide antibiotic',
                          'definition': 'A chemically diverse class of '
                                        'peptides that exhibit antimicrobial '
                                        'properties.',
                          'parents': ['CHEBI:16670']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 1373,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.93234100135318}


"""
Classifies: CHEBI:18254 ribonucleoside
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect

def is_ribonucleoside(smiles: str):
    """
    Determines if a molecule is a ribonucleoside (nucleoside containing D-ribose).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a ribonucleoside, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for ribose substructure
    ribose_smarts = "[C@@H]1O[C@H](CO)[C@@H](O)[C@H]1O"
    ribose_alt_smarts = "[C@@H]1O[C@H](CO)[C@H](O)[C@H]1O"
    ribose_pattern = Chem.MolFromSmarts(ribose_smarts)
    ribose_alt_pattern = Chem.MolFromSmarts(ribose_alt_smarts)
    
    has_ribose = mol.HasSubstructMatch(ribose_pattern) or mol.HasSubstructMatch(ribose_alt_pattern)
    
    if not has_ribose:
        # Check for 3'-deoxy or 2'-O-methyl variants
        deoxy_smarts = "[C@@H]1O[C@H](CO)C[C@H]1O"
        methoxy_smarts = "[C@@H]1O[C@H](CO)[C@@H](OC)[C@H]1O"
        
        deoxy_pattern = Chem.MolFromSmarts(deoxy_smarts)
        methoxy_pattern = Chem.MolFromSmarts(methoxy_smarts)
        
        if not (mol.HasSubstructMatch(deoxy_pattern) or mol.HasSubstructMatch(methoxy_pattern)):
            return False, "No ribose sugar moiety found"

    # Check for nucleobase
    purine_smarts = "c1ncnc2[nH]cnc12" # Adenine/Guanine core
    pyrimidine_smarts = "c1cn([C,H])c(=O)nc1" # Cytosine/Uracil core
    
    purine_pattern = Chem.MolFromSmarts(purine_smarts)
    pyrimidine_pattern = Chem.MolFromSmarts(pyrimidine_smarts)

    has_base = mol.HasSubstructMatch(purine_pattern) or mol.HasSubstructMatch(pyrimidine_pattern)

    if not has_base:
        # Check for modified bases
        mod_purine_smarts = "c1nc2c([nH]1)ncnc2"  # Alternative purine pattern
        mod_pyrimidine_smarts = "c1cnc(=O)[nH]c1"  # Alternative pyrimidine pattern
        
        mod_purine_pattern = Chem.MolFromSmarts(mod_purine_smarts)
        mod_pyrimidine_pattern = Chem.MolFromSmarts(mod_pyrimidine_smarts)
        
        has_base = mol.HasSubstructMatch(mod_purine_pattern) or mol.HasSubstructMatch(mod_pyrimidine_pattern)
        
        if not has_base:
            return False, "No nucleobase moiety found"

    # Check glycosidic linkage
    if mol.HasSubstructMatch(purine_pattern):
        return True, "Purine ribonucleoside"
    else:
        return True, "Pyrimidine ribonucleoside"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18254',
                          'name': 'ribonucleoside',
                          'definition': 'Any nucleoside where the sugar '
                                        'component is D-ribose.',
                          'parents': ['CHEBI:33838', 'CHEBI:47019']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 100,
    'num_true_negatives': 24902,
    'num_false_negatives': 10,
    'num_negatives': None,
    'precision': 0.038461538461538464,
    'recall': 0.2857142857142857,
    'f1': 0.06779661016949154,
    'accuracy': 0.9956028141989127}


"""
Classifies: CHEBI:142605 medium-chain primary fatty alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_medium_chain_primary_fatty_alcohol(smiles: str):
    """
    Determines if a molecule is a medium-chain primary fatty alcohol (C6-C12).
    Primary fatty alcohols have a single OH group at the end of a straight aliphatic chain.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a medium-chain primary fatty alcohol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of primary alcohol group
    primary_alcohol = Chem.MolFromSmarts("[CH2][OH]")
    matches = mol.GetSubstructMatches(primary_alcohol)
    if len(matches) != 1:
        return False, "Must have exactly one primary alcohol group"
    
    # Get the carbon atom of the CH2OH group
    ch2_idx = matches[0][0]
    
    # Check if molecule is aromatic
    if any(atom.GetIsAromatic() for atom in mol.GetAtoms()):
        return False, "Contains aromatic rings"
        
    # Check for presence of non C,H,O atoms
    for atom in mol.GetAtoms():
        if atom.GetSymbol() not in ['C', 'O', 'H']:
            return False, "Contains elements other than C, H, O"
            
    # Check for carboxylic acids
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[CX3](=O)[OX2H1]")):
        return False, "Contains carboxylic acid group"
        
    # Check for esters
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[#6][CX3](=O)[OX2H0][#6]")):
        return False, "Contains ester group"
        
    # Check for ketones/aldehydes
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[CX3]=[OX1]")):
        return False, "Contains carbonyl group"

    # Check for presence of rings
    ring_info = mol.GetRingInfo()
    if ring_info.NumRings() > 0:
        return False, "Contains rings"
        
    # Check for unsaturation
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[#6]=[#6]")) or \
       mol.HasSubstructMatch(Chem.MolFromSmarts("[#6]#[#6]")):
        return False, "Contains unsaturated bonds"

    # Check for branching
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            neighbors = [n for n in atom.GetNeighbors() if n.GetSymbol() == 'C']
            if len(neighbors) > 2:
                return False, "Contains branched carbons"

    # Count carbons in longest chain containing the primary alcohol
    def get_chain_length(atom, visited):
        if atom.GetIdx() in visited:
            return 0
        visited.add(atom.GetIdx())
        max_length = 0
        for neighbor in atom.GetNeighbors():
            if neighbor.GetSymbol() == 'C':
                length = get_chain_length(neighbor, visited)
                max_length = max(max_length, length)
        return max_length + 1

    ch2_atom = mol.GetAtomWithIdx(ch2_idx)
    chain_length = get_chain_length(ch2_atom, set())

    if chain_length < 6 or chain_length > 12:
        return False, f"Carbon chain length ({chain_length}) outside C6-C12 range"

    # Count number of OH groups
    oh_groups = len(mol.GetSubstructMatches(Chem.MolFromSmarts("[OH]")))
    if oh_groups > 2:  # Allow up to 2 OH groups for diols
        return False, "Too many OH groups"

    return True, f"Medium-chain primary fatty alcohol with {chain_length} carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:142605',
                          'name': 'medium-chain primary fatty alcohol',
                          'definition': 'Any primary fatty alcohol with a '
                                        'chain length between C6 and C12.',
                          'parents': ['CHEBI:142622']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.04545454545454545 is too low.\n'
               "True positives: [('CCCCCC(O)CCCCCCO', 'Medium-chain primary "
               "fatty alcohol with 12 carbons')]\n"
               "False positives: [('OC(CCCCCCCC)CO', 'Medium-chain primary "
               "fatty alcohol with 10 carbons'), ('CC(C)CCCCCCO', "
               "'Medium-chain primary fatty alcohol with 8 carbons'), "
               "('C(CCCCCCC)CCCO', 'Medium-chain primary fatty alcohol with 11 "
               "carbons'), ('CCCCCC[C@H](O)CO', 'Medium-chain primary fatty "
               "alcohol with 8 carbons'), ('OCCC(CC(C)(C)C)C', 'Medium-chain "
               "primary fatty alcohol with 6 carbons'), ('OCCCCCC(C)C', "
               "'Medium-chain primary fatty alcohol with 7 carbons'), "
               "('CCCCCCCCCO', 'Medium-chain primary fatty alcohol with 9 "
               "carbons'), ('CC(CCCCCCCCO)C', 'Medium-chain primary fatty "
               "alcohol with 10 carbons'), ('OCCCC(CCCC)C', 'Medium-chain "
               "primary fatty alcohol with 8 carbons'), ('OCCCC(CCCCC)C', "
               "'Medium-chain primary fatty alcohol with 9 carbons'), "
               "('CC(C)CCCCCCCCCCO', 'Medium-chain primary fatty alcohol with "
               "12 carbons'), ('OCC(CCCCCCCC(O)(C)C)(C)C', 'Medium-chain "
               "primary fatty alcohol with 11 carbons'), ('OC(CCCCC)CCO', "
               "'Medium-chain primary fatty alcohol with 8 carbons'), "
               "('OCC(CCCCCC)(C)C', 'Medium-chain primary fatty alcohol with 8 "
               "carbons'), ('CCCC(O)CCCCCCO', 'Medium-chain primary fatty "
               "alcohol with 10 carbons'), ('CCC(C)CCC(CO)C(C)C', "
               "'Medium-chain primary fatty alcohol with 7 carbons'), "
               "('OC(CCCCO)C', 'Medium-chain primary fatty alcohol with 6 "
               "carbons'), ('CC[C@H](C)CCCCCO', 'Medium-chain primary fatty "
               "alcohol with 8 carbons'), ('OC(CCCC(CCO)C)(C)C', 'Medium-chain "
               "primary fatty alcohol with 8 carbons'), ('CC(C)CCCC(C)CO', "
               "'Medium-chain primary fatty alcohol with 7 carbons'), "
               "('CCCCC(CC)CO', 'Medium-chain primary fatty alcohol with 6 "
               "carbons'), ('CCCCCC[C@@H](O)CO', 'Medium-chain primary fatty "
               "alcohol with 8 carbons'), ('OCC(CCCCCCCC)CCCCCC', "
               "'Medium-chain primary fatty alcohol with 10 carbons'), "
               "('C(CCCCCCC)CCO', 'Medium-chain primary fatty alcohol with 10 "
               "carbons'), ('CCCCCCCCCCCCO', 'Medium-chain primary fatty "
               "alcohol with 12 carbons'), ('OCCC(CCCC(C)C)C', 'Medium-chain "
               "primary fatty alcohol with 8 carbons'), ('CCCCCCC(CO)CCCC', "
               "'Medium-chain primary fatty alcohol with 8 carbons'), "
               "('CCCCCCC(CCCCCC)CO', 'Medium-chain primary fatty alcohol with "
               "8 carbons'), ('CC(C)CCCC(C)CCCC(C)CCO', 'Medium-chain primary "
               "fatty alcohol with 12 carbons'), ('CC(O)CCCCCCCCO', "
               "'Medium-chain primary fatty alcohol with 10 carbons'), "
               "('OCCC[C@H](CC)C', 'Medium-chain primary fatty alcohol with 6 "
               "carbons'), ('OCC(CCCCCCCCCC)CC', 'Medium-chain primary fatty "
               "alcohol with 12 carbons'), ('CCCCCCCC(CCO)O', 'Medium-chain "
               "primary fatty alcohol with 10 carbons'), ('C(CCCCCC)O', "
               "'Medium-chain primary fatty alcohol with 7 carbons'), "
               "('CC(C)CCC[C@H](C)CO', 'Medium-chain primary fatty alcohol "
               "with 7 carbons'), ('CCCC(O)C(CC)CO', 'Medium-chain primary "
               "fatty alcohol with 6 carbons'), ('CC(C)CCC[C@@H](C)CO', "
               "'Medium-chain primary fatty alcohol with 7 carbons'), "
               "('CCC[C@@H](C)CCCO', 'Medium-chain primary fatty alcohol with "
               "7 carbons'), ('CCCCCCCCO', 'Medium-chain primary fatty alcohol "
               "with 8 carbons'), ('CCCCCCC(O)CO', 'Medium-chain primary fatty "
               "alcohol with 8 carbons'), ('CCCCCCO', 'Medium-chain primary "
               "fatty alcohol with 6 carbons')]\n"
               "False negatives: [('OCCCCCCCCCCCCO', 'Must have exactly one "
               "primary alcohol group')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 16,
    'num_true_negatives': 183899,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.058823529411764705,
    'recall': 0.5,
    'f1': 0.10526315789473684,
    'accuracy': 0.9999075670003317}


"""
Classifies: CHEBI:31998 N-acylphytosphingosine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_N_acylphytosphingosine(smiles: str):
    """
    Determines if a molecule is an N-acylphytosphingosine (a ceramide that is phytosphingosine 
    having a fatty acyl group attached to the nitrogen).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an N-acylphytosphingosine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of phytosphingosine backbone
    phytosphingosine_backbone = Chem.MolFromSmiles('CCCCCCCCCCCCCCNC(CO)C(O)C(O)CCC=CCCCCCCCC')
    if not mol.HasSubstructMatch(phytosphingosine_backbone):
        return False, "Molecule does not contain the phytosphingosine backbone"

    # Check for the presence of an acyl group attached to the nitrogen
    acylated_nitrogen = Chem.MolFromSmiles('N(C(=O)C)')
    if not mol.HasSubstructMatch(acylated_nitrogen):
        return False, "Nitrogen is not N-acylated"

    # Check the length of the acyl chain
    acyl_chain_length = Descriptors.LipinskiHBACount(mol)
    if acyl_chain_length < 14 or acyl_chain_length > 32:
        return False, "Acyl chain length is not between 14 and 32 carbons"

    # Check for the presence of a galactosyl or lactosyl group
    galactosyl_group = Chem.MolFromSmiles('C(OC1C(C(C(C(O1)CO)O)O)O)O')
    lactosyl_group = Chem.MolFromSmiles('C(OC1C(C(C(C(O1)OC2C(C(C(C(O2)CO)O)O)O)O)O)O)O')
    if not (mol.HasSubstructMatch(galactosyl_group) or mol.HasSubstructMatch(lactosyl_group)):
        return False, "Molecule does not contain a galactosyl or lactosyl group"

    return True, "Molecule is an N-acylphytosphingosine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:31998',
                          'name': 'N-acylphytosphingosine',
                          'definition': 'A ceramide that is phytosphingosine '
                                        'having a fatty acyl group attached to '
                                        'the nitrogen.',
                          'parents': ['CHEBI:139051']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183820,
    'num_false_negatives': 11,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999401624317987}


"""
Classifies: CHEBI:22654 asparagine derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Fragments import fr_Asn

def is_asparagine_derivative(smiles: str):
    """
    Determines if a molecule is an asparagine derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an asparagine derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for asparagine substructure
    asn_pattern = Chem.MolFromSmarts('NC(=O)CC(N)C(=O)O')
    if not mol.HasSubstructMatch(asn_pattern):
        return False, "No asparagine core structure found"
    
    # Get match atoms for asparagine pattern
    matches = mol.GetSubstructMatches(asn_pattern)
    
    for match in matches:
        # Get the relevant atoms from the match
        amide_n = mol.GetAtomWithIdx(match[0])  # Terminal amide N
        alpha_n = mol.GetAtomWithIdx(match[3])  # Alpha amino N
        carboxyl_c = mol.GetAtomWithIdx(match[5])  # Carboxyl C
        
        # Check for modifications at amino group
        if len(list(alpha_n.GetNeighbors())) > 2:
            return True, "Modified at alpha amino group"
            
        # Check for modifications at carboxyl group
        if len(list(carboxyl_c.GetNeighbors())) > 2:
            return True, "Modified at carboxyl group"
            
        # Check for modifications at carboxamide
        if len(list(amide_n.GetNeighbors())) > 2:
            return True, "Modified at carboxamide group"
            
    # Additional check for heteroatom substitutions
    for atom in mol.GetAtoms():
        if atom.GetSymbol() not in ['C', 'H', 'O', 'N']:
            return True, f"Contains heteroatom substitution ({atom.GetSymbol()})"
            
    # Check if it's just unmodified asparagine
    if fr_Asn(mol) == 1 and mol.GetNumAtoms() == 13:  # Asparagine has 13 atoms
        return False, "Unmodified asparagine"
        
    return True, "Asparagine derivative detected"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22654',
                          'name': 'asparagine derivative',
                          'definition': 'An amino acid derivative resulting '
                                        'from reaction of asparagine at the '
                                        'amino group, the carboxy group, or '
                                        'the carboxamide, or from the '
                                        'replacement of any hydrogen of '
                                        'asparagine by a heteroatom. The '
                                        'definition normally excludes peptides '
                                        'containing asparagine residues.',
                          'parents': ['CHEBI:83821']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'fr_Asn' from 'rdkit.Chem.Fragments' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/Fragments.py)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:192499 anthoxanthin
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_anthoxanthin(smiles: str):
    """
    Determines if a molecule is an anthoxanthin (flavonoid pigment).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an anthoxanthin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Basic flavonoid core structure patterns
    flavonoid_core = [
        'O=C1CC(c2ccccc2)Oc2ccccc21',  # flavone
        'O=C1C(O)=C(c2ccccc2)Oc2ccccc21',  # flavonol
        'O=C1CC(Oc2ccccc2)c2ccccc21',  # isoflavone
        'O=C1C=C(c2ccccc2)Oc2ccccc21'  # basic chromone
    ]
    
    # Convert SMARTS patterns to RDKit molecules
    patterns = [Chem.MolFromSmarts(pattern) for pattern in flavonoid_core]
    
    # Check for basic flavonoid structure
    has_flavonoid_core = False
    for pattern in patterns:
        if pattern is not None and mol.HasSubstructMatch(pattern):
            has_flavonoid_core = True
            break
            
    if not has_flavonoid_core:
        return True, "Not a flavonoid structure"

    # Count hydroxyl groups
    oh_pattern = Chem.MolFromSmarts('[OH]')
    num_oh = len(mol.GetSubstructMatches(oh_pattern))

    # Count methoxy groups
    ome_pattern = Chem.MolFromSmarts('OC')
    num_ome = len(mol.GetSubstructMatches(ome_pattern))

    # Count carbonyls
    carbonyl_pattern = Chem.MolFromSmarts('C(=O)')
    num_carbonyls = len(mol.GetSubstructMatches(carbonyl_pattern))

    # Check for glycosylation
    glycoside_pattern = Chem.MolFromSmarts('[OH]C1OC(CO)C(O)C(O)C1O')
    is_glycosylated = mol.HasSubstructMatch(glycoside_pattern)

    # Check for aromatic rings
    ring_info = mol.GetRingInfo()
    aromatic_rings = sum(1 for ring in ring_info.AtomRings() 
                        if all(mol.GetAtomWithIdx(i).GetIsAromatic() for i in ring))

    # Specific criteria for anthoxanthin
    if (aromatic_rings >= 2 and 
        num_carbonyls >= 1 and 
        (num_oh + num_ome) >= 1):
        
        if is_glycosylated:
            return False, "Glycosylated anthoxanthin"
        elif num_oh >= 2:
            return False, "Anthoxanthin with multiple hydroxyl groups"
        elif num_ome >= 1:
            return False, "Anthoxanthin with methoxy substitution"
        else:
            return False, "Basic anthoxanthin structure"
    
    return True, "Not an anthoxanthin structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:192499',
                          'name': 'anthoxanthin',
                          'definition': 'are a type of flavonoid pigments in '
                                        'plants. Anthoxanthins are '
                                        'water-soluble pigments which range in '
                                        'color from white or colorless to a '
                                        'creamy to yellow, often on petals of '
                                        'flowers.',
                          'parents': ['CHEBI:47916']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('O1C2(C3C(C(C3C4=C1C=C(O)C(=C4)C5OC=6C(C(=O)C5)=C(O)C=C(O)C6)(C)C)CC2)C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('COc1ccc(cc1)[C@@H]1CC(=O)c2c(O)c(C)c(O)c(C)c2O1', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('Oc1ccccc1Cc1ccc(O)c(Cc2ccc(O)c(Cc3c(O)c(Cc4cc(Cc5ccccc5O)ccc4O)c4O[C@@H](CC(=O)c4c3O)c3ccccc3)c2)c1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C=CC(O)=C2O)C3=CC=C(OC)C=C3', 'Anthoxanthin "
               "with mixed hydroxyl/methoxy substitution'), "
               "('O1C(CC2=C3OC(CC(=O)C3=C(OC)C=C12)C4=CC=CC=C4)C(C)=C', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('C12=C(C(CC(C3=CC=C(O)C=C3)(O1)O)=O)C=CC(=C2)O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('COc1cc(O)c(C[C@@H](CC=C(C)C)C(C)=C)c2O[C@@H](CC(=O)c12)c1ccc(O)cc1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C=2C1=CC(OC(=O)C3=CC=CC=C3)=CC2O)C4=CC=CC=C4', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1C(CC(=O)C2=C1C(OC)=C(O)C=C2O)C3=CC(O)=C(O)C=C3', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('OC(=O)CCC[C@@H](\\\\C=C\\\\c1ccccc1)c1c(O)cc2O[C@H](CC(=O)c2c1O)c1ccccc1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O([C@H]1C(O)[C@@H](O)C(O[C@H]1OC2=CC=C(C3OC4=C(C(=O)C3)C(O)=C(C=C4)CC=C(C)C)C=C2)C)[C@@H]5OC[C@@H](O)[C@H](O)C5O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C([C@@H](O)[C@H](O)C(O)[C@@H]1OC=2C=C(C3OC=4C(C(=O)C3)=C(O)C=C(OC)C4)C=CC2OC)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C(CC=C(C)C)=C(O)C=C2)C3=CC=CC=C3', 'Basic "
               "anthoxanthin structure'), "
               "('O1C(C1CC=2C=C(C3OC=4C(C(=O)C3)=C(O)C=C(O)C4CC=C(C)C)C=CC2O)(C)C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('C1C(OC2=CC(=CC(=C2C1=O)O)O)C3=CC(=C(C=C3)O)O', 'Anthoxanthin "
               "with multiple hydroxyl groups'), "
               "('O1C(C(CC2=C1C=C(O)C3=C2OC(CC3=O)C4=C(O)C=CC=C4O)CC=C(C)C)(C)C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(=C)C(O)CC\\\\C(C)=C\\\\Cc1c(O)cc2O[C@@H](CC(=O)c2c1O)c1ccc(O)c(O)c1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(C)=CCc1c(O)cc(O)c2C(=O)C[C@H](Oc12)c1ccc(O)cc1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@@H](CC(=O)C2=C1C(O)=C(O)C=C2O)C3=CC(CC=C(C)C)=C(O)C=C3', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('Oc1cc(O)c2C(=O)C[C@H](Oc2c1)c1ccc(O)c(O)c1', 'Anthoxanthin "
               "with multiple hydroxyl groups'), "
               "('Cc1c(O)c(C)c2O[C@@H](CC(=O)c2c1O)c1cc(O)ccc1O[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C(OC)=C(OC)C=C2O)C=3C(OC)=CC=CC3O', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('CC1[C@@H]([C@@H](C([C@@H](O1)OCC2[C@H]([C@H](C([C@@H](O2)OC3=CC(=C4C(=O)C[C@H](OC4=C3)C5=CC(=C(C=C5)OC)O)O)O)O)O)O)O)O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@@H]([C@@H](O)[C@H](O)[C@@H](O)[C@@H]1OC2=CC=3O[C@@H](CC(=O)C3C=C2)C4=CC=C(O[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O)CO)C=C4)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O=C1C2=C(O)C(=C(OC)C=C2O[C@@H](C1)C3=C(O)C=C(O)C=C3)C[C@@H](C(=C)C)CC=C(C)C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C=C(O)C=C2)C3=CC=C(O)C=C3', 'Anthoxanthin "
               "with mixed hydroxyl/methoxy substitution'), "
               "('O1[C@@H]([C@@H](O)[C@H](O)[C@@H](O)[C@@H]1OC2=CC=C(C3OC=4C(C(=O)C3)=C(O)C=C(O)C4)C=C2)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O=C1C2=C(O)C(=C(O)C=C2O[C@H](C1)C3=CC=C(O)C=C3)CC4=C(O)C=CC=C4', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O(C1C(O)C(O)C(OC1CO)OC2=CC=3OC(CC(=O)C3C(OC)=C2)C4=CC(O)=C(O)C=C4)C5OC(C(O)C(O)C5O)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('Cc1c(O)cc(O)c2C(=O)C[C@H](Oc12)c1ccccc1O', 'Anthoxanthin "
               "with multiple hydroxyl groups'), "
               "('OC1C(OC2=C(C=CC(O)=C2)C1=O)C1=CC=C(O)C(O)=C1', 'Anthoxanthin "
               "with multiple hydroxyl groups'), "
               "('O(C1C(O)C(O)C(OC1OC2=CC=3OC(CC(=O)C3C=C2)C4=CC=C(O)C=C4)CO)C5OCC(O)(C5O)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(C)=CCC[C@]1(C)CCc2c(ccc(O)c2O1)[C@@H]1CC(=O)c2c(O1)cc(O)c(CC=C(C)C)c2O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(=C)[C@H](CC=C(C)C)Cc1c(O)cc(O)c2C(=O)C[C@H](Oc12)c1ccc(O)cc1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(C)=CCOc1ccc(cc1)[C@@H]1CC(=O)c2c(O)cc(O)cc2O1', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('C1(C(C(C2(C(C1=O)C(C3=C(O2)C(=C(C4=C3OC(CC4=O)C5=CC=CC=C5)O)C)C(C)C)O)(C)C)=O)(C)C', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1C(C(O)C(=O)C2=C1C=C(O)C(OC)=C2O)C3=CC=C(OC)C=C3', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@@](CC(=O)C=2C1=CC(O)=CC2O)(C=3C=C(C/C=C(/CCC=C(C)C)\\\\C)C(O)=CC3O)[H]', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('Cc1c(O)c(C)c2O[C@H]([C@H](O)C(=O)c2c1O)c1ccccc1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@H]([C@H](O)[C@@H](O)[C@@H](O)[C@@H]1OC=2C=C3OC(CC(=O)C3=C(O)C2)C4=CC=C(OC)C=C4)C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@](C(O)[C@@H](O)[C@H](O)C1CO)(C2=C(O)C3=C(O[C@@H](CC3=O)C4=CC(O)=C(O)C=C4)C=C2O)[H]', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(C1COC=2C=C3OC(CC(=O)C3=C(O)C2)C4=CC=CC=C4)(C)C', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1[C@@H](CC(=O)C2=C1C3=C(OC(C=C3)(C)C)C=C2)C4=CC(=C(O)C=C4)CC=C(C)C', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('OC1C(Oc2cc(O)cc(O)c2C1=O)c1cc(O)c(O)c(O)c1', 'Anthoxanthin "
               "with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C3=C(OC(C=C3)(C)C)C(=C2O)CC=C(C)C)C4=CC5=C(OC(C=C5)(C)C)C=C4', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1[C@]2(CC(C[C@]1(O)/C=C/C3=CC=CC=C3)C=4C=5O[C@H](CC(=O)C5C(O)=CC4OC(=O)CCC2)C6=CC=CC=C6)[H]', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('CC(C)=CCc1cc2C(=O)C[C@H](Oc2c(CC=C(C)C)c1O)c1ccc2OC(C)(C)C=Cc2c1', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('C=1(C=C2OC(C(C(C2=C(C1)O)=O)O)C=3C=CC(=CC3)O)OC4C(C(C(C(O4)CO)O)O)O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O[C@@H]1CO[C@@H](O[C@@H]2[C@H](Oc3cc(O)cc(O)c3C2=O)c2ccc(O)c(O)c2)[C@H](O)[C@H]1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('OC[C@H]1O[C@@H](OC2=CC3=C(C(=O)C[C@H](O3)C3=CC(O)=C(O)C=C3)C(O)=C2)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('Cc1c(O)c2C(=O)C[C@H](Oc2c(C)c1O[C@@H]1O[C@H](COC(=O)c2ccc(O)cc2)[C@@H](O)[C@H](O)[C@H]1O)c1cc(O)ccc1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@@H](OC=2C([C@]3(OC4=C(C(=O)C3)C(O)=CC(OC)=C4OC)[H])=C(OC)C=CC2)C(O)[C@@H](O)[C@H](O)C1C(O)=O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C=2C1=CC(O)=CC2O)C3=CC=C(OC)C=C3', 'Anthoxanthin "
               "with mixed hydroxyl/methoxy substitution'), "
               "('O[C@@H]1[C@H](Oc2cc([O-])cc(O)c2C1=O)c1ccc(O)c(O)c1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C([C@@H](O)[C@H](O)C(O)[C@@H]1OC2=C(C=3O[C@@H](CC(=O)C3C(O)=C2)C4=CC=C(O)C=C4)CC=C(C)C)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O[C@@H]1[C@H](Oc2cc(O)cc(O)c2C1=O)c1ccc(O)c(O)c1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O(C1C(O)C(O)C(OC1CO)OC=2C3=C(OC(C(O)C3=O)C4=CC=CC=C4)C=C(O)C2)C5OC(C(O)C(O)C5O)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(C)=CCc1cccc([C@@H]2CC(=O)c3c(O)cc4OC(C)(C)C=Cc4c3O2)c1O', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1C(CC(=O)C2=C1C=C(O)C(CCC(O)(C)C)=C2O)C3=CC(O)=C(O)C=C3', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O=C1C2=C(O[C@@H](C1)C3=CC=CC=C3)C(=C(O)C=C2O)C/C=C(/CCC=C(C)C)\\\\C', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1C2=C(C(CC1(C3=CC=C(C=C3)O)O)=O)C(=CC(=C2)O)O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('OC[C@H]1O[C@H]([C@H](O)[C@@H](O)[C@@H]1O)c1c(O)cc(O)c2C(=O)C[C@H](Oc12)c1ccc(O)cc1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@]2(O)[C@@]([C@H](C3=C1C(=C(O)C4=C3O[C@@H](CC4=O)C5=CC=CC=C5)C)C6=CC=CC=C6)(C(=O)C(C(=O)C2(C)C)(C)C)[H]', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('Cc1c(O)c2C(=O)C[C@H](Oc2c(C)c1O[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)c1cc(O)ccc1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1[C@@H](CC(=O)C2=C1C(OC)=C(OC)C(=C2O)CC3=C(O)C=CC(OC)=C3)C4=CC=CC=C4', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('S(OC1=C(OC)C=C(C2OC=3C(C(=O)C2O)=C(O)C=C(O)C3)C=C1)(O)(=O)=O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(C2=CC(=C(O)C(=C2)CC=C(C)C)CC=C(C)C)CC(=O)C3=C1C=C(O)C=C3', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('COc1cc(ccc1O)[C@@H]1CC(=O)c2c(O)cc(O)cc2O1', 'Anthoxanthin "
               "with multiple hydroxyl groups'), "
               "('O1C(OC2C(OC=3C(C2=O)=C(O)C=C(O)C3)C4=CC(OC5OC(C(O)C(O)C5O)C(O)=O)=CC=C4)C(O)C(O)C1CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(OC2=C(C3C(OC=4C(C3=O)=C(O)C=C(O)C4)C5=CC=C(O)C=C5)C=6OC(CC(=O)C6C(O)=C2)C7=CC(O)=C(O)C=C7)C(O)C(O)C(O)C1CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C=2C1=C(CC=C(C)C)C(O)=CC2OC)C3=CC=C(O)C=C3', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1C([C@H](O)[C@H](O)C(O)[C@@H]1OC=2C=C3OC(CC(=O)C3=C(O)C2)C4=CC=CC=C4)C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O(C1C(O)C(O)C(OC1OC2=CC=C(C3OC4=C(C(=O)C3)C=CC(OC5OC(C(O)C(O)C5O)C(O)=O)=C4)C=C2)CO)C6OCC(O)(C6O)COC(=O)C=7C=8C(NC7)=CC=CC8', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(C(O)C(O)C(O)C1OC=2C3=C(OC(CC3=O)C4=CC(O)=C(OC)C=C4)C=C(OC)C2)CO', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC1=C[C@H]([C@@H]([C@H](C1)c1ccc(O)cc1O)C(=O)c1ccc(O)cc1O)c1c(O)ccc([C@H]2Oc3cc(O)ccc3C(=O)[C@@H]2O)c1O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(C)=CCc1c(O)c(CC=C(C)C)c2O[C@@H](CC(=O)c2c1O)c1cc(O)cc(O)c1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(OC=2C=C3OC(CC(=O)C3=C(O)C2)C4=CC=C(OC)C=C4)C(O)C(O)C(O)C1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('COc1cc(OC)c2C(=O)C[C@H](Oc2c1C=O)c1ccccc1', 'Anthoxanthin "
               "with mixed hydroxyl/methoxy substitution'), "
               "('COc1cc(O)c2C(=O)C[C@H](Oc2c1CC=C(C)C)c1cc2C=CC(C)(C)Oc2cc1O', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('COc1cc(O)c(C)c2O[C@@H](CC(=O)c12)c1ccccc1', 'Anthoxanthin "
               "with mixed hydroxyl/methoxy substitution'), "
               "('O1C(C(OC)C(=O)C2=C1C=C(O)C(OC)=C2O)C3=CC(O)=C(OC)C=C3', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('[H][C@@]1(CC(=O)c2c(O)cc3O[C@]4(C[C@]([H])(c5c(O)cc(O[C@@H]6O[C@H](CO)[C@@H](O)[C@H](O)[C@H]6O)cc5O4)c3c2O1)c1ccc(O)c(O)c1)c1ccc(O)c(O)c1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('Oc1cc(O)c2C(=O)C[C@H](Oc2c1)c1cc(O)c(O)c(O)c1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CCC=C(C)C)(C=CC=2C1=CC=3OC(CC(=O)C3C2O)C4=C(O)C=C(O)C=C4)C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(OC2=C(C3OC4=C(C(=O)C3)C=CC(O)=C4)C(O)=C(C(OC)=C2)CC=C(C)C)C(O)C(O)C(O)C1C(O)=O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('COc1c(O)cc2OC(CC(=O)c2c1O)c1ccccc1', 'Anthoxanthin with "
               "mixed hydroxyl/methoxy substitution'), "
               "('O1[C@@H](CC(=O)C2=C1C(O)=C(O)C=C2O)C3=CC=C(OC)C=C3', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C([C@@H](OC(=O)C2=CC(O)=C(O)C(O)=C2)[C@H](O)C(O)[C@@H]1OC3=C(C=4O[C@@H](CC(=O)C4C(O)=C3C)C5=CC=C(OC)C=C5)C)COC(=O)C6=CC(O)=C(O)C(O)=C6', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C=C(O)C=C2)C3=CC(O)=C(O)C=C3', 'Anthoxanthin "
               "with multiple hydroxyl groups'), "
               "('O1C(OC2=C(O)C=C(C3OC4=C(C5C(OC=6C(C5=O)=C(O)C=C(O)C6)C7=CC=C(O)C=C7)C(O)=CC(O)=C4C(=O)C3)C=C2)C(O)C(O)C(O)C1C(O)=O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O[C@@H]1[C@H](Oc2cc([O-])cc(O)c2C1=O)c1ccc(O)cc1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('Oc1ccccc1Cc1c(O)cc2O[C@@H](CC(=O)c2c1O)c1ccccc1', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(C=CC=2C=3OC(CC(=O)C3C(O)=C(C12)CC=C(C)C)C4=CC=C(O)C=C4)(C)C', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('C1=2C(C([C@@]3([C@](O1)(C=4C(O3)=CC(=CC4)O)CC=C(C)C)O)=O)=C(C5=C(C2)OC(C=C5)(C)C)O', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('CC(C)=CCc1c(O)cc(O)c2C(=O)C[C@H](Oc12)c1cc(ccc1O)C(C)(C)C=C', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C3=C(OC(C=C3)(C)C)C=C2)C4=CC=C(OC)C=C4', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1[C@@](CC(=O)C2=C1C=C(O)C(=C2O)CC=C(C)C)(C=3C(O)=CC(O)=C(OC)C3)[H]', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(C(C=2C=3OC=CC(=O)C3C(O)=CC2O)C(=O)C=4C1=CC(O)=CC4O)C5=CC=C(O)C=C5', "
               "'Anthoxanthin with multiple hydroxyl groups'), "
               "('O1C(CC(=O)C2=C1C(=C(O)C(=C2O)CC=C(C)C)C)C3=CC(=C(OC)C=C3)CC=C(C)C', "
               "'Anthoxanthin with mixed hydroxyl/methoxy substitution'), "
               "('O1C(CCC=C(C)C)(C=CC=2C1=CC(O)=C(C3OC=4C(C(=O)C3)=C(O)C=C(O)C4)C2)CO', "
               "'Anthoxanthin with multiple hydroxyl groups')]\n"
               'False negatives: '
               "[('O1C(=C(CC=C(C)C)C(=O)C=2C1=CC(O)=CC2O)C3=C(O)C=C(O)C=C3', "
               "'Not a flavonoid structure'), "
               "('O1C=2C(C3C(C(CC(=C3)C)C4=C(O)C=C(O)C=C4)C(=O)C5=C(O)C=C(O)C=C5)=C(O)C=C(O)C2C(=O)C(=C1C6=C(O)C=C(O)C=C6)CC=C(C)C', "
               "'Not a flavonoid structure'), "
               "('O1C(C2=CC(CCC(CO)C)=C(O)C=C2)=C(OC)C(=O)C3=C1C=C(O)C(OC)=C3O', "
               "'Not a flavonoid structure'), "
               "('COc1cc2oc(cc(=O)c2c(O)c1C)-c1ccc(OC)c(c1)-c1c(O)cc(O)c2c1oc(cc2=O)-c1ccc(O)cc1', "
               "'Not a flavonoid structure'), "
               "('COc1cc(O)c2c(oc(cc2=O)-c2ccc(O)cc2)c1C', 'Not a flavonoid "
               "structure'), "
               "('[H][C@]12SC(C)(C)C(N1C(=O)[C@H]2NC(=O)C1=CC(=CC=C1)C1=CC(=O)C2=CC=CC=C2O1)C(O)=O', "
               "'Not a flavonoid structure'), "
               "('COc1cc(O)c2c(c1)oc(-c1cc(O)c(OC)c(OC)c1)c(OC)c2=O', 'Not a "
               "flavonoid structure'), "
               "('OC[C@H]1O[C@H]([C@H](O)[C@@H](O)[C@@H]1O)c1c(O)c([C@@H]2OC[C@H](O)[C@H](O)[C@H]2O)c(O)c2c1oc(cc2=O)-c1ccc(O)c(O)c1', "
               "'Not a flavonoid structure'), "
               "('OC[C@@]1(O)CO[C@@H](OC[C@H]2O[C@H]([C@H](O)[C@@H](O)[C@@H]2O)c2c(O)ccc3c2occ(-c2ccc(O)cc2)c3=O)[C@@H]1O', "
               "'Not a flavonoid structure'), "
               "('COc1cc(ccc1O)-c1oc2cc(O[C@H]3O[C@@H](CO)[C@H](O)[C@@H](O)[C@@H]3O)cc(O)c2c(=O)c1O', "
               "'Not a flavonoid structure'), "
               "('[H][C@]1(OC[C@@H](O)[C@H](O)[C@H]1O)O[C@@H]1[C@@H](O)[C@H](C)O[C@@H](OC[C@H]2O[C@@H](Oc3c(oc4cc(O)cc(O)c4c3=O)-c3ccc(O)c(O)c3)[C@H](O)[C@@H](O)[C@@H]2O)[C@@H]1O', "
               "'Not a flavonoid structure'), "
               "('CC(=O)O[C@@H]1[C@@H](O)[C@H](O)[C@@H](CO)O[C@H]1Oc1c(oc2cc(O[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)cc(O)c2c1=O)-c1ccc(O)cc1', "
               "'Not a flavonoid structure'), "
               "('O1C(C2=CC(CC=C(C)C)=C(O)C=C2)=CC(=O)C3=C1C=C(O)C=C3', 'Not a "
               "flavonoid structure'), "
               "('C[C@@H]1O[C@H](Oc2c(oc3cc(O)cc(O)c3c2=O)-c2ccc(O)c(O)c2)[C@@H](O)[C@H](O)[C@@H]1O', "
               "'Not a flavonoid structure'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\Cc1c(O)cc(O)c2c1oc(cc2=O)-c1ccccc1', "
               "'Not a flavonoid structure'), "
               "('C[C@@H]1O[C@@H](Oc2cc(O)c3c(c2)oc(-c2ccc(O)cc2)c(O)c3=O)[C@H](OC(=O)\\\\C=C\\\\c2ccc(O)cc2)[C@H](OC(=O)\\\\C=C\\\\c2ccc(O)cc2)[C@H]1O', "
               "'Not a flavonoid structure'), "
               "('OC[C@H]1O[C@@H](Oc2cc3oc(-c4ccc(O)c(O)c4)c(O)c(=O)c3c(O)c2Cc2ccc(O)cc2)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Not a flavonoid structure'), "
               "('O1C(C=CC=2C1=C(C=3OC=4C(C(=O)C3CC=C(C)C)=C(O)C=C(O)C4)C=CC2O)(C)C', "
               "'Not a flavonoid structure'), "
               "('C1(OC2OC(C(C(C2O)O)OC(/C=C/C=3C=C(C(=CC3)O)OC)=O)CO)=CC(=C4C(OC(=CC4=O)C5=CC=C(O)C(=C5)C6OC(O)C(C6CO)O)=C1)O', "
               "'Not a flavonoid structure'), ('O=c1cc(oc2ccccc12)-c1ccccc1', "
               "'Not a flavonoid structure'), "
               "('CC(=C)C1Cc2c(O1)cc1oc(-c3ccc(O)c(O)c3)c(O)c(=O)c1c2O', 'Not "
               "a flavonoid structure'), "
               "('OC[C@@H]1O[C@H](Oc2c(oc3cc(O)cc(O)c3c2=O)-c2cc(O)c(O)c(O)c2)[C@@H](O)[C@H](O)[C@@H]1O', "
               "'Not a flavonoid structure'), "
               "('O1C(C2=CC(CC=C(C)C)=C(OC)C=C2)=C(OC)C(=O)C3=C1C=C(O)C(OC)=C3O', "
               "'Not a flavonoid structure'), "
               "('CC(C)=CCc1c(O)c(CC=C(C)C)c2oc(cc(=O)c2c1O)-c1ccccc1', 'Not a "
               "flavonoid structure'), "
               "('OC[C@H]1O[C@@H](Oc2cc3oc(cc(=O)c3c(O)c2[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)-c2ccc(O)cc2)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Not a flavonoid structure'), "
               "('C1(=O)C=C(C2=CC=C(C=C2)O)OC3=C1C(O)=C(C(=C3)OC)O', 'Not a "
               "flavonoid structure'), "
               "('COc1cc(cc(OC)c1OC)-c1cc(=O)c2c(OC)c(OC)c(OC)c(OC)c2o1', 'Not "
               "a flavonoid structure'), "
               "('COc1ccc(OC)c(c1)-c1cc(=O)c2c(OC)c(OC)c(OC)c(OC)c2o1', 'Not a "
               "flavonoid structure'), "
               "('O1C=2C(C(=O)C=C1C3=CC=CC=C3)=C(O)C=CC2', 'Not a flavonoid "
               "structure'), "
               "('COc1cc(\\\\C=C\\\\C(=O)OC[C@H]2OC(Oc3cc(O)c4c(c3)oc(cc4=O)-c3cc(OC)c(O)c(OC)c3)[C@H](O)[C@@H](O)[C@@H]2O)cc(OC)c1O', "
               "'Not a flavonoid structure'), "
               "('COc1cc(ccc1O)-c1cc(=O)c2c(O)cc(O[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O[C@@H]3O[C@@H](C)[C@H](O)[C@@H](O)[C@H]3O)cc2o1', "
               "'Not a flavonoid structure'), "
               "('Oc1ccc(c(O)c1)-c1oc2cc(O)cc(O)c2c(=O)c1O', 'Not a flavonoid "
               "structure'), ('COc1cc(OC)c2c(c1)oc(-c1ccccc1)c(OC)c2=O', 'Not "
               "a flavonoid structure'), "
               "('COc1ccc(cc1O)-c1cc(=O)c2c(O)cc(O[C@@H]3O[C@H](CO[C@@H]4O[C@@H](C)[C@H](O)[C@@H](O)[C@H]4O)[C@@H](O)[C@H](O)[C@H]3O)cc2o1', "
               "'Not a flavonoid structure'), "
               "('S(OC=1C=C(C=2OC3=C(C(=O)C2)C=CC(O)=C3)C=CC1)(O)(=O)=O', 'Not "
               "a flavonoid structure'), "
               "('CN1CCCCC1c1c(O)cc(O)c2c1oc(cc2=O)-c1ccccc1', 'Not a "
               "flavonoid structure'), "
               "('COc1cc(O)cc2oc(-c3cc(O)c(O)c(O)c3)c(O[C@@H]3O[C@@H](C)[C@H](O)[C@@H](O)[C@H]3O)c(=O)c12', "
               "'Not a flavonoid structure'), "
               "('Oc1ccc2c(c1)oc(-c1ccc(O)c(O)c1)c(O)c2=O', 'Not a flavonoid "
               "structure'), "
               "('COc1cc(ccc1O)-c1cc(=O)c2c(O)c([C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)c(O[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O[C@@H]3O[C@@H](C)[C@H](O)[C@@H](O)[C@H]3O)cc2o1', "
               "'Not a flavonoid structure'), "
               "('CC(C)=CCc1ccc(cc1O)-c1oc2c(CC=C(C)C)c(O)c(CC=C(C)C)c(O)c2c(=O)c1O', "
               "'Not a flavonoid structure'), "
               "('[H][C@]1(O[C@H](COC(=O)\\\\C=C\\\\c2ccc(O)cc2)[C@@H](O)[C@H](O)[C@H]1O)O[C@H]1[C@@H](O[C@@H](C)[C@H](O)[C@H]1O)Oc1c(oc2cc(O[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)cc(O)c2c1=O)-c1ccc(O)c(O)c1', "
               "'Not a flavonoid structure'), "
               "('O1C2=C(C(O)=C(CC=C(CO)C)C(O)=C2)C(=O)C=C1C3=C(O)C=C(O)C=C3', "
               "'Not a flavonoid structure'), "
               "('Oc1cc(O)c2c(c1)oc(-c1ccc(O)c(OS(O)(=O)=O)c1)c(OS(O)(=O)=O)c2=O', "
               "'Not a flavonoid structure'), "
               "('Oc1cc(O)c2c(c1)oc(cc2=O)-c1cc(O)c(O)cc1O', 'Not a flavonoid "
               "structure'), "
               "('O=C1C(O)=C(OC2=C1C(O)=CC(=C2CC=C(C)C)O)C3=CC=C(OC)C=C3', "
               "'Not a flavonoid structure'), "
               "('C1=2C(C=C(OC1=CC(=CC2O)O)C=3C=C(OC)C(=C(C3)OC)O[C@H]([C@@H](C4=CC(OC)=C(C=C4)O)O)CO[C@H]5[C@@H]([C@H]([C@@H]([C@H](O5)CO)O)O)O)=O', "
               "'Not a flavonoid structure'), "
               "('O1C(C(O)CC2=C1C=C3OC(=CC(=O)C3=C2O)C4=CC=C(O)C=C4)(C)C', "
               "'Not a flavonoid structure'), "
               "('C[C@@H]1O[C@@H](OC[C@H]2O[C@@H](Oc3c(oc4cc(O)c(O)c(O)c4c3=O)-c3ccc(O)cc3)[C@H](O)[C@@H](O)[C@@H]2O)[C@H](O)[C@H](O)[C@H]1O', "
               "'Not a flavonoid structure'), "
               "('[C@H]1(O[C@H]([C@@H]([C@H]([C@H]1O)O)O)C)O[C@H]2[C@@H](O[C@@H]([C@H]([C@@H]2O)O)CO)OC3=CC4=C(C(C=C(O4)C5=CC(=C(C(=C5)OC)O)OC)=O)C(=C3)O', "
               "'Not a flavonoid structure'), "
               "('O1C(=C(CC=C(C)C)C(=O)C=2C1=CC(O)=CC2O)C=3C(OC)=CC(OC)=C(O)C3', "
               "'Not a flavonoid structure'), "
               "('C1(=C(C(=C(C2=C1OC(=C(C2=O)*)C3=C(C(=C(C(=C3*)*)*)[H])*)*)*)*)*', "
               "'Not a flavonoid structure'), "
               "('COc1cc2oc(-c3ccc(O)cc3O)c(CC=C(C)C)c(=O)c2c(O)c1\\\\C=C\\\\C(C)C', "
               "'Not a flavonoid structure'), "
               "('O1C([C@@H](O)[C@@H](O)[C@H](O)C1O)COC2=C(OC=3C(C2=O)=C(O)C=C(O)C3)C4=CC(O)=C(O)C=C4', "
               "'Not a flavonoid structure'), "
               "('C1=2C(OC(=CC1=O)C3=CC(=C(C=C3)O)O)=CC(=C(C2O)[C@H]4[C@@H]([C@H]([C@H](CO4)O)O)O[C@H]5[C@@H]([C@H]([C@@H]([C@@H](CO)O5)O)O)O)O', "
               "'Not a flavonoid structure'), "
               "('COc1cc(OC)c(cc1OC)-c1cc(=O)c2c(OC)c(OC)c(OC)c(OC)c2o1', 'Not "
               "a flavonoid structure'), "
               "('CN1CCCC1c1c(O)cc2oc(cc(=O)c2c1O)-c1ccccc1', 'Not a flavonoid "
               "structure'), "
               "('[H][C@]1(O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)c1c(O)c(O)c2oc(cc(=O)c2c1O)-c1ccc(O)cc1', "
               "'Not a flavonoid structure'), "
               "('Cc1c(oc2cc(O)cc(O)c2c1=O)-c1ccc(O)c(O)c1', 'Not a flavonoid "
               "structure'), "
               "('COc1cc(O)c2c(c1)oc(-c1cc(OC)c(OC)c(OC)c1)c(OC)c2=O', 'Not a "
               "flavonoid structure'), "
               "('O([C@H]1[C@@H]([C@H]([C@@H]([C@@H](COC(=O)/C=C/C2=CC(=C(C=C2)O)OC)O1)O)O)O)[C@@H]3[C@H]([C@@H]([C@H](O[C@H]3C4=C(C=C5C(=C4O)C(C=C(O5)C6=CC=C(C(=C6)OC)O)=O)O)CO)O)O', "
               "'Not a flavonoid structure'), "
               "('S(OCC1(OC2=C(C(O)=C3C(OC(=C(C3=O)C/C=C(\\\\CCC=C(C)C)/C)C4=C(O)C=C(O)C=C4)=C2)C=C1)C)(O)(=O)=O', "
               "'Not a flavonoid structure'), "
               "('OC[C@H]1O[C@@H](Oc2c(oc3cc(O)cc(O)c3c2=O)-c2ccc(O)cc2)[C@H](O)[C@H](O)[C@@H]1O', "
               "'Not a flavonoid structure'), "
               "('O[C@H]1[C@H](Oc2c(oc3cc(O)cc(O)c3c2=O)-c2cc(O)c(O)c(O)c2)O[C@@H]([C@@H](O)[C@@H]1O)C(O)=O', "
               "'Not a flavonoid structure'), "
               "('O[C@@H]1[C@@H](COC(=O)\\\\C=C\\\\c2ccc(O)cc2)O[C@@H](Oc2c(oc3cc(O)cc(O)c3c2=O)-c2ccc(O)cc2)[C@H](O)[C@H]1O', "
               "'Not a flavonoid structure'), "
               "('COc1c(oc2cc(O[C@@H]3O[C@H](CO[C@@H]4O[C@@H](CO)[C@H](O)[C@H]4O)[C@@H](O)[C@H](O)[C@H]3O)cc(O)c2c1=O)-c1ccc(O)cc1', "
               "'Not a flavonoid structure'), "
               "('COc1ccc(cc1)-c1cc(=O)c2c(O)c(OC)c(O)c(OC)c2o1', 'Not a "
               "flavonoid structure')]",
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 66,
    'num_false_positives': 100,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.39759036144578314,
    'recall': 1.0,
    'f1': 0.5689655172413793,
    'accuracy': 0.39759036144578314}


"""
Classifies: CHEBI:143212 thienopyrimidine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_thienopyrimidine(smiles: str):
    """
    Determines if a molecule is a thienopyrimidine (pyrimidine ring fused to thiophene).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a thienopyrimidine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Generate the ring information
    rings = mol.GetRingInfo()
    
    # Find all 6-membered rings (potential pyrimidine)
    six_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            # Check if ring has exactly 2 nitrogens
            n_count = sum(1 for atom in atoms if atom.GetSymbol() == 'N')
            if n_count == 2:
                six_rings.append(ring)
                
    if not six_rings:
        return False, "No pyrimidine ring found"

    # Find all 5-membered rings (potential thiophene)
    five_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 5:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            # Check if ring has exactly 1 sulfur
            s_count = sum(1 for atom in atoms if atom.GetSymbol() == 'S')
            if s_count == 1:
                five_rings.append(ring)
                
    if not five_rings:
        return False, "No thiophene ring found"

    # Check for fusion between pyrimidine and thiophene rings
    for six_ring in six_rings:
        for five_ring in five_rings:
            # Find shared atoms between rings
            shared = set(six_ring).intersection(set(five_ring))
            if len(shared) == 2:  # Two shared atoms indicates fusion
                # Verify ring atoms
                six_atoms = [mol.GetAtomWithIdx(i) for i in six_ring]
                five_atoms = [mol.GetAtomWithIdx(i) for i in five_ring]
                
                # Check pyrimidine ring
                n_atoms = [a for a in six_atoms if a.GetSymbol() == 'N']
                if len(n_atoms) != 2:
                    continue
                    
                # Check thiophene ring
                s_atoms = [a for a in five_atoms if a.GetSymbol() == 'S']
                if len(s_atoms) != 1:
                    continue
                    
                return True, "Contains fused thiophene-pyrimidine ring system"
                
    return False, "No fused thiophene-pyrimidine ring system found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:143212',
                          'name': 'thienopyrimidine',
                          'definition': 'A class of aromatic heterobicyclic '
                                        'compounds each of which contains a '
                                        'pyrimidine ring ortho fused to a '
                                        '5-membered thiophene ring.',
                          'parents': [   'CHEBI:27171',
                                         'CHEBI:38101',
                                         'CHEBI:38106']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 72026,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.9986136719670609}


"""
Classifies: CHEBI:26606 sapogenin
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors
from rdkit.Chem import AllChem

def is_sapogenin(smiles: str):
    """
    Determines if a molecule is a sapogenin (organic polycyclic compound, aglycon moiety of a saponin).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a sapogenin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for organic compound
    if not AllChem.GetMolDescriptor(mol, 'isOrganic'):
        return False, "Not an organic compound"

    # Check for polycyclic structure
    num_rings = rdMolDescriptors.CalcNumRings(mol)
    if num_rings < 2:
        return False, "Not a polycyclic compound"

    # Check for steroid or triterpenoid skeleton
    ring_info = mol.GetRingInfo()
    ring_sizes = [len(ring) for ring in ring_info.AtomRings()]

    is_steroid = (
        len(set(ring_sizes)) == 2
        and 6 in ring_sizes
        and 5 in ring_sizes
    )

    is_triterpenoid = (
        len(set(ring_sizes)) == 2
        and 6 in ring_sizes
        and 9 in ring_sizes
    )

    if not (is_steroid or is_triterpenoid):
        return False, "Not a steroid or triterpenoid skeleton"

    return True, "Molecule is a sapogenin"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26606',
                          'name': 'sapogenin',
                          'definition': 'Any organic polycyclic compound that '
                                        'is the aglycon moiety of a saponin; '
                                        'sapogenins may be steroids or '
                                        'triterpenoids.',
                          'parents': ['CHEBI:51958']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.AllChem' has no attribute 'GetMolDescriptor'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:22562 anilines
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_anilines(smiles: str):
    """
    Determines if a molecule is an aniline (aromatic amine with at least one amino substituent on benzene).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an aniline, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find all aromatic rings
    rings = mol.GetRingInfo()
    aromatic_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms) and all(atom.GetSymbol() == 'C' for atom in atoms):
                aromatic_rings.append(ring)

    if not aromatic_rings:
        return False, "No aromatic benzene rings found"

    # Check for amino substituents on benzene rings
    for ring in aromatic_rings:
        ring_atoms = set(ring)
        for atom_idx in ring:
            atom = mol.GetAtomWithIdx(atom_idx)
            for neighbor in atom.GetNeighbors():
                if neighbor.GetIdx() not in ring_atoms:
                    # Check if neighbor is NH2 or substituted NH
                    if neighbor.GetSymbol() == 'N':
                        # Count non-H neighbors of N
                        non_h_neighbors = len([n for n in neighbor.GetNeighbors() if n.GetSymbol() != 'H'])
                        total_neighbors = neighbor.GetDegree()
                        h_count = total_neighbors - non_h_neighbors
                        
                        if h_count >= 1 or neighbor.GetNumExplicitHs() >= 1:
                            substituents = []
                            for n in neighbor.GetNeighbors():
                                if n.GetIdx() not in ring_atoms:
                                    substituents.append(n.GetSymbol())
                            
                            if substituents:
                                return True, f"Aniline with substituted amino group. Substituents: {', '.join(set(substituents))}"
                            else:
                                return True, "Primary aniline (unsubstituted amino group)"

    return False, "No amino substituents found on benzene ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22562',
                          'name': 'anilines',
                          'definition': 'Any  aromatic amine that is benzene '
                                        'carrying at least one amino '
                                        'substituent and its substituted '
                                        'derivatives.',
                          'parents': ['CHEBI:22712', 'CHEBI:33860']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 2311,
    'num_false_negatives': 39,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.943265306122449}


"""
Classifies: CHEBI:21437 Mo-molybdopterin cofactor
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_Mo_molybdopterin_cofactor(smiles: str):
    """
    Determines if a molecule is a Mo-molybdopterin cofactor.

    A Mo-molybdopterin cofactor is defined as a molybdopterin cofactor in which
    the coordinated metal is a mononuclear molybdenum or molybdate species.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a Mo-molybdopterin cofactor, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a molybdenum atom
    has_mo = any(atom.GetSymbol() == 'Mo' for atom in mol.GetAtoms())
    if not has_mo:
        return False, "No molybdenum atom found"

    # Check for the presence of a molybdopterin scaffold
    molybdopterin_scaffold = Chem.MolFromSmarts('[#7]1[#6]2[#6]([#6]([#6]([#6]2[#7][#6]1[#7])[#6])[#7])[#6]')
    if mol.GetSubstructMatch(molybdopterin_scaffold) == []:
        return False, "Molybdopterin scaffold not found"

    # Check for coordination of molybdenum to the molybdopterin scaffold
    mo_atom = next((atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'Mo'), None)
    if mo_atom is None:
        return False, "No molybdenum atom found"

    scaffold_atoms = mol.GetSubstructMatch(molybdopterin_scaffold)
    for atom_idx in scaffold_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        if atom in mo_atom.GetNeighbors():
            return True, "Mo-molybdopterin cofactor identified"

    return False, "Molybdenum not coordinated to molybdopterin scaffold"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:21437',
                          'name': 'Mo-molybdopterin cofactor',
                          'definition': 'A molybdopterin cofactor in which the '
                                        'coordinated metal is a mononuclear '
                                        'molybdenum or molybdate species.',
                          'parents': ['CHEBI:25372', 'CHEBI:35202']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183922,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9999945629421008}


"""
Classifies: CHEBI:139051 phytoceramide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_phytoceramide(smiles: str):
    """
    Determines if a molecule is a phytoceramide based on structural features.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a phytoceramide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for amide group (R-C(=O)-N-R)
    amide_pattern = Chem.MolFromSmarts('[CX3](=O)[NX3]')
    if not mol.HasSubstructMatch(amide_pattern):
        return False, "No amide group found"

    # Check for phytosphingoid base core structure
    # Looking for -CH(OH)-CH(OH)-CH(NH-)-CH2OH or similar pattern
    sphingoid_pattern = Chem.MolFromSmarts('[CH1,CH2]([OH1])[CH1]([OH1])[CH1]([NH1])[CH2][OH1]')
    if not mol.HasSubstructMatch(sphingoid_pattern):
        return False, "No phytosphingoid base core structure found"

    # Check for long alkyl chain (fatty acid part)
    # Count carbons in longest chain from amide
    amide_matches = mol.GetSubstructMatches(amide_pattern)
    if amide_matches:
        amide_carbon = amide_matches[0][0]  # Get carbon atom index of first amide match
        fatty_acid_carbons = 0
        visited = set()
        
        def count_chain_carbons(atom_idx, current_chain=0):
            nonlocal fatty_acid_carbons
            if atom_idx in visited:
                return
            visited.add(atom_idx)
            atom = mol.GetAtomWithIdx(atom_idx)
            if atom.GetSymbol() == 'C':
                current_chain += 1
                fatty_acid_carbons = max(fatty_acid_carbons, current_chain)
                for neighbor in atom.GetNeighbors():
                    if neighbor.GetSymbol() == 'C':
                        count_chain_carbons(neighbor.GetIdx(), current_chain)
                        
        count_chain_carbons(amide_carbon)
        
        if fatty_acid_carbons < 12:  # Typical fatty acids have 12+ carbons
            return False, "Fatty acid chain too short"

    # Optional: Check for sugar moiety (common in phytoceramides but not required)
    sugar_pattern = Chem.MolFromSmarts('[CH1]1[OH1,OR][CH1][CH1][CH1][CH1]([CH2][OH1,OR])O1')
    has_sugar = mol.HasSubstructMatch(sugar_pattern)

    if has_sugar:
        return True, "Phytoceramide with sugar moiety"
    else:
        return True, "Phytoceramide without sugar moiety"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:139051',
                          'name': 'phytoceramide',
                          'definition': 'A class of phytoceramides obtained by '
                                        'formal condensation of the carboxy '
                                        'group of any fatty acid with the '
                                        'amino group of any phytosphingoid '
                                        'base.',
                          'parents': ['CHEBI:83273', 'CHEBI:84403']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 7,
    'num_true_negatives': 183762,
    'num_false_negatives': 10,
    'num_negatives': None,
    'precision': 0.46153846153846156,
    'recall': 0.375,
    'f1': 0.41379310344827586,
    'accuracy': 0.9999075006121283}


"""
Classifies: CHEBI:19643 2-hydroxyisophthalic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import BRICS

def is_2_hydroxyisophthalic_acid(smiles: str):
    """
    Determines if a molecule is a 2-hydroxyisophthalic acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a 2-hydroxyisophthalic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Identify the ring system
    ring_info = mol.GetRingInfo()
    ring_atoms = set()
    for ring in ring_info.AtomRings():
        if len(ring) == 6:
            ring_atoms.update(ring)
            break
    else:
        return False, "No 6-membered ring found"

    # Check if the ring is aromatic
    aromatic_ring = all(mol.GetAtomWithIdx(idx).GetIsAromatic() for idx in ring_atoms)
    if not aromatic_ring:
        return False, "Ring is not aromatic"

    # Check if the ring contains only carbon atoms
    if not all(mol.GetAtomWithIdx(idx).GetSymbol() == 'C' for idx in ring_atoms):
        return False, "Ring contains non-carbon atoms"

    # Check for the carboxylic acid groups
    carboxyl_count = sum(Chem.Mol.GetBondBetweenAtoms(mol, idx1, idx2).GetBondType() == Chem.BondType.DOUBLE
                         and mol.GetAtomWithIdx(idx2).GetSymbol() == 'O'
                         and mol.GetAtomWithIdx(idx2).GetIsAromatic() is False
                         and mol.GetAtomWithIdx(idx2).GetTotalDegree() == 1
                         for idx1 in ring_atoms for idx2 in mol.GetAtomWithIdx(idx1).GetNeighbors())
    if carboxyl_count != 2:
        return False, "Molecule does not have two carboxylic acid groups"

    # Check for the hydroxy group at position 2
    hydroxy_idx = None
    for idx in ring_atoms:
        if mol.GetAtomWithIdx(idx).GetTotalNumHs() == 1:
            hydroxy_idx = idx
            break
    if hydroxy_idx is None:
        return False, "No hydroxy group found on the ring"

    # Check if the hydroxy group is at position 2
    brics = BRICS.BRICSDecomposition(mol)
    brics_smarts = brics.GetBrics()
    if 'O=C1OCC=2C1=C(O)C(=CC2C(=O)O)' not in brics_smarts:
        return False, "Hydroxy group is not at position 2"

    return True, "Molecule is a 2-hydroxyisophthalic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:19643',
                          'name': '2-hydroxyisophthalic acid',
                          'definition': 'A hydroxybenzoic acid that is '
                                        'isophthalic acid in which the '
                                        'hydrogen at position 2 is substituted '
                                        'by a hydroxy group.',
                          'parents': ['CHEBI:24676', 'CHEBI:33853']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.GetBondBetweenAtoms(Mol, int, Atom)\n'
             'did not match C++ signature:\n'
             '    GetBondBetweenAtoms(RDKit::ROMol {lvalue} self, unsigned int '
             'idx1, unsigned int idx2)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:131875 hydroperoxyicosatrienoate
"""
from rdkit import Chem

def is_hydroperoxyicosatrienoate(smiles: str):
    """
    Determines if a molecule is a hydroperoxyicosatrienoate anion.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a hydroperoxyicosatrienoate anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for a carboxylate anion group
    carboxylate_present = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetFormalCharge() == -1:
            neighbors = [mol.GetAtomWithIdx(n.GetIdx()).GetSymbol() for n in atom.GetNeighbors()]
            if 'C' in neighbors:
                carboxylate_present = True
                break

    if not carboxylate_present:
        return False, "No carboxylate anion group found"

    # Check for a hydroperoxy group
    hydroperoxy_present = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 1:
            neighbors = [mol.GetAtomWithIdx(n.GetIdx()).GetSymbol() for n in atom.GetNeighbors()]
            if 'O' in neighbors:
                hydroperoxy_present = True
                break

    if not hydroperoxy_present:
        return False, "No hydroperoxy group found"

    # Check for an icosanoid chain (20 carbon atoms)
    carbon_count = sum(atom.GetSymbol() == 'C' for atom in mol.GetAtoms())
    if carbon_count != 20:
        return False, "Molecule does not contain 20 carbon atoms"

    # Check for at least three double bonds
    double_bond_count = sum(bond.GetBondType() == Chem.BondType.DOUBLE for bond in mol.GetBonds())
    if double_bond_count < 3:
        return False, "Molecule does not contain at least three double bonds"

    return True, "Molecule is a hydroperoxyicosatrienoate anion"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131875',
                          'name': 'hydroperoxyicosatrienoate',
                          'definition': 'An icosanoid anion arising from '
                                        'deprotonation of the carboxylic acid '
                                        'function of any '
                                        'hydroperoxyicosatrienoic acid.',
                          'parents': [   'CHEBI:134019',
                                         'CHEBI:57560',
                                         'CHEBI:62937']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.GetAtomWithIdx(Mol, Atom)\n'
               'did not match C++ signature:\n'
               '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int '
               'idx)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 41,
    'num_true_negatives': 183882,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.023809523809523808,
    'recall': 1.0,
    'f1': 0.046511627906976744,
    'accuracy': 0.9997770818381505}


"""
Classifies: CHEBI:25223 methanesulfonate ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_methanesulfonate_ester(smiles: str):
    """
    Determines if a molecule is a methanesulfonate ester.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a methanesulfonate ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all sulfur atoms
    sulfur_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'S']

    # Check if there is exactly one sulfur atom
    if len(sulfur_atoms) != 1:
        return False, "Molecule does not contain exactly one sulfur atom"

    sulfur_atom = mol.GetAtomWithIdx(sulfur_atoms[0])

    # Check if the sulfur atom has four neighbors
    if len(sulfur_atom.GetNeighbors()) != 4:
        return False, "Sulfur atom does not have four neighbors"

    # Check if one neighbor is an oxygen atom with a double bond
    has_double_bonded_oxygen = False
    for neighbor in sulfur_atom.GetNeighbors():
        if neighbor.GetSymbol() == 'O' and mol.GetBondBetweenAtoms(sulfur_atom.GetIdx(), neighbor.GetIdx()).GetBondType() == Chem.BondType.DOUBLE:
            has_double_bonded_oxygen = True
            break

    if not has_double_bonded_oxygen:
        return False, "Sulfur atom does not have a double-bonded oxygen neighbor"

    # Check if another neighbor is an oxygen atom with a single bond
    has_single_bonded_oxygen = False
    for neighbor in sulfur_atom.GetNeighbors():
        if neighbor.GetSymbol() == 'O' and mol.GetBondBetweenAtoms(sulfur_atom.GetIdx(), neighbor.GetIdx()).GetBondType() == Chem.BondType.SINGLE:
            has_single_bonded_oxygen = True
            break

    if not has_single_bonded_oxygen:
        return False, "Sulfur atom does not have a single-bonded oxygen neighbor"

    # Check if the remaining neighbors are carbon atoms
    remaining_neighbors = [neighbor for neighbor in sulfur_atom.GetNeighbors() if neighbor.GetSymbol() != 'O']
    if not all(neighbor.GetSymbol() == 'C' for neighbor in remaining_neighbors):
        return False, "Remaining neighbors of sulfur atom are not carbon atoms"

    return True, "Molecule is a methanesulfonate ester"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25223',
                          'name': 'methanesulfonate ester',
                          'definition': 'An organosulfonic ester resulting '
                                        'from the formal condensation of '
                                        'methanesulfonic acid with the hydroxy '
                                        'group of an alcohol, phenol, '
                                        'heteroarenol, or enol.',
                          'parents': ['CHEBI:48544', 'CHEBI:83347']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 6434,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9846977811782709}


"""
Classifies: CHEBI:22901 bisphenol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_bisphenol(smiles: str):
    """
    Determines if a molecule is a bisphenol compound.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a bisphenol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Check for presence of at least 2 phenol groups
    phenol_pattern = Chem.MolFromSmarts('[OH]c1ccc([#6])cc1')
    phenol_matches = mol.GetSubstructMatches(phenol_pattern)
    
    if len(phenol_matches) < 2:
        return False, "Does not contain at least 2 phenol groups"
    
    # Check if the phenol groups are connected through a central carbon/heteroatom
    # Pattern looks for 2 phenols connected through any atom
    bisphenol_pattern = Chem.MolFromSmarts('[OH]c1ccc([#6]([*])[#6]c2ccc([OH])cc2)cc1')
    bisphenol_matches = mol.GetSubstructMatches(bisphenol_pattern)
    
    if not bisphenol_matches:
        return False, "Phenol groups not properly connected through central atom"
        
    # Get the central connecting atom
    for match in bisphenol_matches:
        # The central atom is at position 5 in the SMARTS pattern
        central_atom = mol.GetAtomWithIdx(match[5])
        central_atom_symbol = central_atom.GetSymbol()
        
        # Count substituents on central atom
        substituents = []
        for neighbor in central_atom.GetNeighbors():
            if neighbor.GetIdx() not in [match[4], match[7]]:  # Exclude phenol carbons
                substituents.append(neighbor.GetSymbol())
                
        if central_atom_symbol == 'C':
            return True, f"Bisphenol with carbon bridge and substituents: {', '.join(substituents)}"
        else:
            return True, f"Bisphenol with {central_atom_symbol} bridge and substituents: {', '.join(substituents)}"
            
    return False, "Structure does not match bisphenol pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22901',
                          'name': 'bisphenol',
                          'definition': 'By usage, the methylenediphenols, '
                                        'HOC6H4CH2C6H4OH, commonly '
                                        'p,p-methylenediphenol, and their '
                                        'substitution products (generally '
                                        'derived from condensation of two '
                                        'equivalent amounts of a phenol with '
                                        'an aldehyde or ketone). The term also '
                                        'includes analogues in the the '
                                        'methylene (or substituted methylene) '
                                        'group has been replaced by a '
                                        'heteroatom.',
                          'parents': ['CHEBI:33853']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 62562,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9983722711604749}


"""
Classifies: CHEBI:141131 acyl-AMP(1-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem, Descriptors, rdMolDescriptors

def is_acyl_AMP_1_(smiles: str):
    """
    Determines if a molecule is an acyl-AMP(1-) ion, defined as an organophosphate oxoanion obtained
    by deprotonation of the phosphate OH group of any acyl-AMP; major species at pH 7.3.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an acyl-AMP(1-) ion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of phosphate group
    has_phosphate = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'P' and atom.GetFormalCharge() == 1:
            has_phosphate = True
            break

    if not has_phosphate:
        return False, "Molecule does not contain a phosphate group"

    # Check for the presence of an acyl group
    has_acyl = False
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.DOUBLE and bond.GetBeginAtom().GetSymbol() == 'C' and bond.GetEndAtom().GetSymbol() == 'O':
            has_acyl = True
            break

    if not has_acyl:
        return False, "Molecule does not contain an acyl group"

    # Check for the presence of an AMP moiety
    has_amp = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'N' and atom.GetIsAromatic():
            neighbors = [mol.GetAtomWithIdx(n).GetSymbol() for n in atom.GetNeighbors()]
            if 'C' in neighbors and 'N' in neighbors and 'C' in neighbors:
                has_amp = True
                break

    if not has_amp:
        return False, "Molecule does not contain an AMP moiety"

    return True, "Molecule is an acyl-AMP(1-) ion"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:141131',
                          'name': 'acyl-AMP(1-)',
                          'definition': 'An organophosphate oxoanion obtained '
                                        'by deprotonation of the phosphate OH '
                                        'group of any acyl-AMP; major species '
                                        'at pH 7.3.',
                          'parents': ['CHEBI:58945']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "name 'is_acyl_AMP_1__' is not defined",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}